class Exceptions(Exception,):
    measures = ['NA', 'span', 'width']
    myUnits = {}
    layers = {}
    geometry = {}
    width = {}
    thickness = {}
    material = {}
    EG = {}
    results = {}
    y = {}
    a = {}
    z = {}
    span = {}
    loads = {}
    mass = {}
    check = {}
    wall = {}
    def __int__(self):
        Exceptions.__int__(self)

    def __str__(self):
        return 'ALL Exceptions'

    def is_valid(self,
                 numberCheck=False,
                 widthCheck=False,
                 materialCheck=False,
                 thicknessEntriesCheck=False,
                 thicknessEntries=None,
                 thicknessExistsCheck=False,
                 loadExistsCheck=False,
                 spanExistsCheck=False,
                 massExistsCheck=False,
                 resultsExistsCheck=False,

                 ):
        if numberCheck:
            if 'layers' not in self.results.keys():
                raise KeyError('Your CLT PANEL object must be set with object.add_layer(int) method')
        if widthCheck:
            if 'width' not in self.geometry.keys():
                raise KeyError('Your CLT PANEL object must be set with object.add_width(int or float) method')
        if materialCheck:
            if 'E' not in self.material.keys():
                raise KeyError('Your CLT PANEL object must be set with object.add_defined_material(str) method')
        if thicknessEntriesCheck:
            if not len(thicknessEntries) == self.layers['layers']:
                m = self.layers['layers']
                raise ValueError(
                    f'Your CLT PANEL was added {m} Layer, but Entered {len(thicknessEntries)} thickness layer')
        if thicknessExistsCheck:
            if 'tk_1' not in self.thickness.keys():
                raise KeyError('Your CLT PANEL object must be set with object.add_thickness(int) method')
        if massExistsCheck:
            if 'mass' not in self.geometry.keys():
                raise KeyError('Your CLT PANEL object must be set with object.mass(int) method')

        if spanExistsCheck:
            if 'span' not in self.geometry.keys():
                raise KeyError('Your CLT PANEL object must be set with object.span(int) method')

        if loadExistsCheck:
            if 'Q' not in self.loads.keys() or 'G' not in self.loads.keys():
                raise KeyError('Your CLT PANEL object must be set with object.load(int) method')

        if resultsExistsCheck:
            if 'M_y' not in self.results.keys() or 'V_y' not in self.results.keys():
                raise KeyError('Your CLT PANEL object must be set with object.My or Vy(int) method')



import numpy as np
import pandas as pd
import math
from Materials_V3 import Materials
from Exceptions_V3 import Exceptions
from Loading_V3 import Loading
import json

"""
Goals : 
    1 - Omit Dictionary
    2 - Return Dictionary ,Context/Json
    3 - Using Arrays/Pandas
    4 - Exceptions
    5 - Units
    6 - Add Beams


neutral axis : NA : EAY/EA  - AY: First Moment Of Inertia
C : Far AXIS
I = Moment Of Inertia E*bh3/12 + az^2 mm4 =>E*I
EI : effective bending stiffness(I) of the panel for the major strength axis
S = I/C the section modulus in the major direction (EI/E)/NA mm3
Factors : 
    -loadDurationFactor: Load Duration Factor Kd - Default : 1
    -serviceConditionFactor: Service Condition Factor Ks - Default : 1
    -treatmentFactor: Treatment Factor Kt - Default : 1
    -systemFactor: System Factor Kh - Default : 1
    
-Moment-
Element Resistance : fb(radiata moment strength) * S(section) * Resistance Factor * Strength Reduction Factors(Kd...)
Element Force Moment using G and Q = L2/8   Force(Q,G,V) --> Action(moment,shear)
-Shear-
fv(radiata shear strength)
V =  3/2 fv/A (Rectangular Section)

---------------------------
stress = F/A
Deflection : Need EI ,GA , Weight
    - 5WL^4/384EI + kWL2/8(GA)<=L/300
Vibration :  Need EI ,Mass
    - 𝐿 ≤ 0.11(EI/10^6 )^0.29/Mass^0.12
"""


class CLT(Materials, Exceptions, Loading):
    """Floor CLt PANEL CALCULATION is the main class of cross-wood package"""
    results = {}
    wall = {}

    def __init__(self):
        """This Class calculate CLT FLOOR PANEL"""
        Materials.__init__(self)
        Exceptions.__init__(self)
        Loading.__init__(self)

    def __str__(self):
        return self.results

    def add_layer(self, number: int):
        self.results.clear()
        self.results.update({'layers': number})

    def add_thickness(self, *args):
        self.is_valid(numberCheck=True)
        self.results.update({'geometry': {'width': 1000}})
        width = self.results['geometry']['width']
        self.results.update({'thickness': args})
        tk, eay, ea, e, g, a, y, z, ei = [], [], [], [], [], [], [], [], []
        tk = args
        status = True
        for i in range(0, self.results['layers']):
            a.append(tk[i] * width)
            y.append(tk[i] / 2 + sum(self.results['thickness'][:i]))
            if status is True:
                e.append(self.results['material']['E'])
                g.append(self.results['material']['G'])
                status = False
            else:
                e.append(self.results['material']['E'] / 15)
                g.append(self.results['material']['E'] / 16)
                status = True
            eay.append(e[i] * a[i] * y[i])
            ea.append(e[i] * a[i])
        self.results.update({'H': sum(args)})
        self.results.update({'a': a})
        self.results.update({'Ag': sum(a)})
        self.results.update({'y': y})
        self.results.update({'E': e})
        self.results.update({'G': g})
        na = sum(eay) / sum(ea)
        self.results.update({'NA': na})
        for i in range(0, self.results['layers']):
            z.append(abs(self.results['NA'] - y[i]))
            ei.append(e[i] * width * args[i] ** 3 / 12 + a[i] * z[i] ** 2)
        self.results.update({'z': z})
        self.results.update({'EI': round(sum(ei), 2)})
        # PANDAS
        index = range(1, len(args) + 1)
        data = {'tk': tk, 'E': e, 'G': g, 'A': a, 'Y': y, 'EAY': eay, 'EA': ea, 'Z': z, }
        df = pd.DataFrame(index=index, data=data, )
        ea = df.sum(axis='index')['EA']
        # print(df)
        # print(f"EA  is  : {ea}")
        # print(df.loc[df.tk == 42, 'Y'], 'nist')

    def floor_calculation(self, resistanceFactor=0.9,
                          strengthModificationFactor=0.85,
                          loadDurationFactor=1,
                          serviceConditionFactor=1,
                          treatmentFactor=1,
                          systemFactor=1,
                          ):
        width = self.results['geometry']['width']
        # Moment
        s = (self.results['EI'] / self.results['material']['E']) / self.results['NA']
        f_b = self.results['material']['fb'] * (
                loadDurationFactor * serviceConditionFactor * treatmentFactor * systemFactor)
        m_y = resistanceFactor * f_b * s * strengthModificationFactor
        self.results.update({'Fb': round(f_b, 2), 'S': round(s, 2), 'M': round(m_y, 2)})
        # Vy :Flat-wise Shear Resistance
        shear = []
        val = 0
        for i in range(1, len(self.results['thickness'])):
            val += self.results['thickness'][i] / self.results['G'][i] * width
        shear.append(self.results['H'])
        shear.append(self.results['thickness'][0])
        shear.append(self.results['thickness'][len(self.results['thickness']) - 1])
        shear.append(shear[1] / self.results['G'][0] * width)
        shear.append(shear[2] / self.results['G'][len(self.results['G']) - 1] * width)
        shear.append(val)
        ga = (shear[0] - shear[1] / 2 + shear[2] / 2) ** 2 / (shear[3] + shear[4] + shear[5])
        fs = self.results['material']['fs']
        f_s = fs * loadDurationFactor * serviceConditionFactor * treatmentFactor * systemFactor
        v = resistanceFactor * f_s * 2 * self.results['Ag'] / 3
        self.results.update({'GA': round(ga, 2), 'Fs': f_s, 'V': round(v, 2)})

    def wall_calculation(self, *args, **kwargs):
        width = self.results['geometry']['width']
        e = self.results['material']['E']
        ei = self.results['EI']
        l_eff = self.results['geometry']['length']
        f_c = self.results['material']['fc']
        i_eff = ei / e
        h, a = [], []
        for layer in args:
            h.append(self.results['thickness'][layer - 1])
            a.append(self.results['thickness'][layer - 1] * width)
        a = sum(a)
        r_eff = math.sqrt(i_eff / a)
        c_c = l_eff / (math.sqrt(12) * r_eff)
        k_zc = 6.3 * (2 * math.sqrt(3) * r_eff * l_eff) ** -.13
        k_se = 1
        k_t = 1
        k_c = (1 + (f_c * k_zc * c_c ** 3) / (35 * e * k_se * k_t)) ** -1
        p = 0.8 * f_c * a * k_zc * k_c
        self.results.update({'wall': {'H_eff': sum(h),
                                      'A_eff': a,
                                      'I_eff': i_eff,
                                      'r_eff': r_eff,
                                      'C_c': c_c,
                                      'k_zc': k_zc,
                                      'k_c': k_c,
                                      'P': p
                                      }})

    def Deflection(self):
        length = self.results['geometry']['length']
        dead = self.results['loads']['G']
        live = self.results['loads']['Q']
        ei = self.results['EI']
        ga = self.results['GA']
        top = (5 * (dead + live) * length ** 4) / (384 * ei)
        down = ((dead + live) * length ** 2) / (8 * ga)
        deflection = top + down
        self.results.setdefault('Deflection', round(deflection, 2))

    def Vibration(self):
        ei = self.results['EI']
        w = self.results['geometry']['weight']
        h = self.results['H']
        vibration = .11 * (ei / 1000000) ** .29 / ((h / 1000 * w) ** .12)
        self.results.setdefault('Vibration', round(vibration, 2))


clt = CLT()
clt.add_layer(6)
clt.add_defined_material('radiata')
clt.add_new_material(species='radiata', grade='radiata', fb=10, fc=10, ft=10, fs=10, fp=10, E=1000, G=1000)
clt.add_thickness(42, 20, 42, 20, 42, 20)
clt.floor_calculation()
clt.loading(G=1.5, Q=1, width=1983, length=12345, weight=1367)
clt.Deflection()
clt.Vibration()
clt.wall_calculation(1, 3, 5)
print(json.dumps(clt.results, indent=4))



class UnitMixins:
    result = {}
    measures = ['NA', 'span', 'width']
    measuresUnits = ['mm', 'cm', 'm']

    def add_unit(self, myDict: dict, **kwargs):
        for key, value in kwargs.items():
            if key in myDict and key in self.measures:
                if value == 'm':
                    myDict.setdefault(key + '_' + value, myDict[str(key)] / 1000)
                    myDict.pop(key)
                elif value == 'mm':
                    myDict.setdefault(key + '_' + value, myDict[str(key)])
                    myDict.pop(key)
                elif value == 'cm':
                    myDict.setdefault(key + '_' + value, myDict[str(key)] / 10)
                    myDict.pop(key)
            else:
                continue
        return myDict

    def centimeter(self, value):
        return self

    def millimeter(self, value):
        return self



class Loading:
    results = {}

    def __init__(self):
        pass

    def loading(self, G=1.5, Q=0.5, P=None, width=1000, length=1000, weight=500):
        self.results['geometry'].update({'width': width, 'length': length, 'weight': weight})
        width = self.results['geometry']['width']
        length = self.results['geometry']['length']
        weight = self.results['geometry']['weight']

        comb_1 = 1.35 * G
        m_n_1 = comb_1 * length ** 2 / 8
        v_n_1 = comb_1 * length / 2
        comb_2 = 1.2 * G + 1.5 * Q
        m_n_2 = comb_2 * length ** 2 / 8
        v_n_2 = comb_2 * length / 2
        long_service = G + 0.7 * Q
        short_service = G + 0.4 * Q

        self.results.setdefault('loads', {'G': G,
                                          'Q': Q,
                                          'P': P,
                                          'Mn1': round(m_n_1, 2),
                                          'Mn2': round(m_n_2, 2),
                                          'Vn1': round(v_n_2, 2),
                                          'Vn2': round(v_n_2, 2),
                                          })

        if v_n_1 >= self.results['V'] or v_n_2 >= self.results['V']:
            self.results['loads']['V_check'] = 'not satisfied'
        else:
            self.results['loads']['V_check'] = 'satisfied'

        if m_n_1 >= self.results['M'] or m_n_2 >= self.results['M']:
            self.results['loads']['M_check'] = 'not satisfied'
        else:
            self.results['loads']['M_check'] = 'satisfied'
        return self.results['loads']



"""
mass :The mass of an object is the same everywhere. Weight:  Weight depends on the effect of gravity(W=m∗g)
Density : Kg/m3  - multiply density by volume to get mass
E : Elasticity : GPa
Strength MPa(*10=Kg/cm2) : Material  - Resistance : civil engineering Material
Pa :N/m2 , MPa :N/mm2 GPa:KN/mm2
Kg = kg * g = N
floor : 500 kg/m2 , 5 KN/m2 ,0.5 Ton/m2
"""


class Materials:
    defiendMaterials = [
        {'radiata': {'species': 'radiata_pine', 'grade': 'building', 'fb': 33.9, 'fc': 28.6, 'ft': 20.3, 'fs': 4.1,
                     'fp': 9.4, 'E': 12, 'G': 0.8, 'Density': 500}},
        {'douglas': {'species': 'douglas_fir', 'grade': 'building', 'fb': 33, 'fc': 30.1, 'ft': 19.8, 'fs': 3.2,
                     'fp': 9.7, 'E': 12, 'G': 0.8, 'Density': 500}},
        {'steel': {'Density': 7850, 'E': 320, 'strength': 240}},
        {'concrete': {'Density': 2500, 'E': 25, 'strength': 25, 'grade': 350}},

    ]
    material = {}
    results = {}

    def __int__(self):
        pass

    def add_defined_material(self, materialName: str):
        found = False
        for material in self.defiendMaterials:
            for key, value in material.items():
                if key == materialName:
                    # self.material = value
                    self.results.update({'material': value})
                    print(f'material is set with {key}')
                    found = True
                    break
        if not found:
            print("No stores were found.")

    def add_new_material(self, species: str, grade: str, fb: float,
                         fc: float, ft: float, fs: float, fp: float, E: int, G: int):
        self.results.update({'material': {'species': species,
                                          'grade': grade,
                                          'fb': fb,
                                          'fc': fc,
                                          'ft': ft,
                                          'fs': fs,
                                          'fp': fp,
                                          'E': E,
                                          'G': G
                                          }})



import cadwork as cw
import utility_controller as uc
import element_controller as ec
import scene_controller as sc
import geometry_controller as gc
import material_controller as mc
import attribute_controller as ac
import shop_drawing_controller as sdc
import list_controller as lc
import menu_controller as mec
import visualization_controller as vc
drill_list=[]
panel_list=[]
massage ="yes"
element_ids = ec.get_all_identifiable_element_ids()
for element in element_ids:
    if ac.is_panel(element):
        panel_list.append(element)
    if ac.is_drilling(element):
        drill_list.append(element)

for panel in panel_list:
    for drill in drill_list:
        if ec.check_if_elements_are_in_collision(panel, drill):
            cw.set_auto_attribute([panel], str(drill))



""""

Iran : C = ABI/Ru
New Zealand :
AI =>> Z * Ru * N(D,T)
B =>> Ch(T)
Ru = Sp/Ku
As a Result :
Ch(T) = Spectral Shape Factor
ELASTIC SITE SPECTRA FOR HORIZONTAL LOADING => C(T1) = Ch(T) * Z * Ru * N(D,T)
Horizontal Design Action Coefficient  => Cd(T1) = C(T1) * Sp/Ku

Step 1 :
importance level: Importance level for hotel, offices, and apartments less than 15 stories high car buildings, shopping
    centers less than 10000 m^2 gross area is 2
Step 2 :
For this importance level (2): consequences of failure are ordinary, thus; Annual probability of the design event
    for safety is 1/500  for wind, 1/150 for snow and 1/500 for earthquake

Step 3 :
    Determine Vertical irregularity

Step 4:
    Determine method of analysis
    based on irregularity, floors and period; or (The equivalent static method of analysis shall be used only when at
    least one of the following criteria is satisfied)
•	The height between the base and the top of the structure is less than 10 m
•	The largest translational period calculated as specified in Clause 4.1.2(4.1.2 Period determination for the
    equivalent static method) is less than 0.4 s or
•	The structure is not classified as irregular under Clause 4.5 (4.5 STRUCTURAL IRREGULARITY) and the largest
    translation period is less than 2.0 seconds
•	6.2.3 Scaling of deflections
•	5.3.2 Accidental eccentricity


Step 5:
    Estimate the natural period   T1=  N/20=5/20=0.25     Where:N= the number of storeys

Step 6:
    Determine subsoil class
    	Class A – Strong rock 3.3.3.2
    	Class B – Rock 3.1.3.3
    	Class C – Shallow soil sites 3.1.3.4
    	Class D – Deep or soft soil sites 3.1.3.5
Step 7:
    Determine The spectral shape factor,
    Ch (T) based on the site subsoil class and natural period (T=N/20) defined in Clause 3.1.3
    	The spectral shape factor functions are graphed in Figure 3.1 for general cases and in Figure 3.2 for values for
    the modal response spectrum and the numerical integration time history methods
    	 The spectral shape factor for the equivalent static method need not exceed the value given by equation 3.1(2) for
    a period T of 0.4 s.

Step 8:
    Determine Z: the hazard factor determined from Clause 3.1.4
    Auckland is 0.13

Step 9:
    R: the return period factor R_δ or R_u for the appropriate limit state determined from Clause 3.1.5
    but limited such that ZR_u does not exceed 0.7    From=Table 3.5   1⁄500 =1  ultimate state,1⁄25=serviceability

Step 10:
    N (T, D): the near-fault factor determined from Clause 3.1.6
    Annual probability of exceedance≥1⁄250   ,    N(T,D)=1
    Annual probability of exceedance<1⁄250
        N(T,D)=N_max (T)     D≤2 Km
        N(T,D)=1+(N_max (T)-1)(20-D)/18     2Km<D≤2 Km if T<1.5 then N (T, D) =1
        N(T,D)=1    D>2 Km
    D : the shortest distance (in kilometers) from the site to the nearest fault listed in Table 3.6.

Step 11:
    Determine 3.1 ELASTIC SITE SPECTRA FOR HORIZONTAL LOADING  C(T1) = Ch(T) * Z * Ru * N(D,T)

Step 12:
    18.	Determine 2.2 STRUCTURAL TYPES

Step 13:
    Determine μ= MAXIMUM LEVELS OF DUCTILITY DEMAND ON STRUCTURAL STEEL SEISMIC-RESISTING SYSTEMS

Step 14: Sp and Ku
    μ= MAXIMUM LEVELS OF DUCTILITY DEMAND ON STRUCTURAL STEEL SEISMIC-RESISTING SYSTEMS
    Sp : The structural performance factor determined by Clause 4.4
    the structural performance factor, Sp, for the ultimate limit state shall be taken
    as 0.7 except where 1.0 < < 2.0 then Sp shall be defined by: Sp = 1.3 – 0.3

    Ku : The structural performance
    For soil classes A, B, C, D Kμ =μ   For T_1≥0.7s , Kμ =((μ-1) T_1)/0.7 For T1<0.7s
    For soil classes E      Kμ =μ   For T1≥1s or μ<1.5 , Kμ = (μ-1.5)T1+1.5 T1<1s or μ≥1.5


Step 15:
    Horizontal Design Action Coefficient  => Cd(T1) = C(T1) * Sp/Ku

Step 16:

    4.2 SEISMIC WEIGHT AND SEISMIC MASS w_i=G_i+∑▒φ_e  Q_i (وزن لرزه ایی)
        G_i,φ_e Q_i  Are summed between the mid-heights of adjacent stores
        G_i : The permanent action (self-weight or 'dead' action) at level i (DEAD)
        φ_e : 0. 6 is the earthquake-imposed action (live load) combination factor for storage applications
        φ_e : 0. 3 is the earthquake-imposed action (live load) combination factor for all
        Q_i : The imposed action for each occupancy class on level i (AS/NZS 1170.1)
        Q_i For roofs shall include allowance of 1kPa for ice on roofs
        the seismic mass at each level,m_i shall be taken as w_i/g


Step 17:
    Determine 6.2.1.2 Horizontal seismic shear V=Cd(T1)Wt
Step 18:
    Determine 3.2 SITE HAZARD SPECTRA FOR VERTICAL LOADING Cv(T)=0.7C(T)

Step 19:
    Determine 5.4 VERTICAL DESIGN ACTIONS Cvd=Cv(T) * Sp




"""
import math


class HDAC:
    results = {}
    pass
